import docker

from os import mkdir

from . import errors
from .. import container


class ContainerManager():
    def __init__(self) -> None:
        self.client = docker.from_env()
        self.bot_containers = {}
        try:
            mkdir(f'/tmp/dockerfiles')
        except FileExistsError:
            pass

    def add(self, context: str, name: str, token: str) -> None:
        if (not self.bot_exists(name)):
            self.bot_containers[name] = container.Container(
                self.client, context, name, token)
            self.bot_containers[name].dockerize()
        else:
            raise(ReferenceError(
                f"Container with name '{name}' already exists"))

    async def start(self, name: str) -> None:
        try:
            await self.get_bot_by_name(name).start()
        except errors.BotReferenceError as e:
            raise(e('Impossible to start bot '))

    def stop(self, name: str) -> None:
        if (self.bot_exists(name)):
            if (self.bot_containers[name].status == container.Container.STATUS_RUNNING):
                self.bot_containers[name].stop()
            else:
                raise(errors.BotStatusError("bot is not running"))
        else:
            raise(KeyError(f'No container with name {name}'))

    def bot_exists(self, name: str) -> bool:
        return True if name in self.bot_containers.keys() else False

    def get_online_bots(self) -> list:
        return [bot.name for bot in self.bot_containers.values() if bot.status == container.Container.STATUS_RUNNING]

    def get_bot_by_name(self, name: str) -> container.Container:
        if (self.bot_exists(name) and self.bot_containers[name].is_container_valid()):
            return self.bot_containers[name]
        raise(errors.BotReferenceError(f'No container with name \'{name}\''))



import docker
import docker.errors
import docker.client

from distutils.dir_util import copy_tree
from os import mkdir


from .. import manager
from . import errors
from . import status


class Container():

    def __init__(self, client, path: str, name: str, token: str):
        self.client = client
        self.ctn = None
        self.path = path
        self.name = name
        self.ctn_name = 'ddx-' + name
        self.docker_context = f'/tmp/dockerfiles/{self.ctn_name}'
        self.status = self.STATUS_EXITED
        self.img_name = f'{self.name}_img'
        self.token = token

    def dockerize(self):
        try:
            mkdir(self.docker_context)
        except FileExistsError:
            pass

        f = open(f'{self.docker_context}/Dockerfile', 'w')
        f.write("FROM python:latest\nCOPY ./app /app \nCOPY ./requirements.txt /requirements.txt \nRUN pip install -r /requirements.txt\nCMD python /app/main.py\n")
        f.close()

        copy_tree(self.path, self.docker_context)

        self.client.images.build(path=self.docker_context, tag=self.img_name)

        try:
            self.client.containers.get(self.ctn_name).remove(force=True)
        except docker.errors.NotFound:
            pass

        self.ctn = self.client.containers.create(
            image=self.img_name,
            name=self.ctn_name,
            environment=[f'BOT_TOKEN={self.token}'],
            network_mode='bridge',
            auto_remove=True)

    def start(self):
        try:
            self.ctn.start()
        except docker.errors.APIError as e:
            raise(e('Api Error'))
        except Exception as e:
            raise(e('Already Started'))
        self.status = self.STATUS_RUNNING

    def stop(self):
        self.ctn.stop()
        self.status = self.STATUS_EXITED

    async def is_running(self) -> bool:
        try:
            self.ctn = self.client.containers.get(self.ctn_name)
            if (self.ctn.status == status.STATUS_RUNNING):
                return True
            else:
                return False
        except docker.errors.NotFound as e:
            raise(manager.errors.ReferenceError(
                f'Impossible to access container : {e}'))

    def is_container_valid(self) -> bool:
        try:
            self.ctn = self.client.containers.get(self.ctn_name)
            return True
        except docker.errors.NotFound as e:
            raise (errors.ValidityError(
                f'Impossible to access container : {e}'))



class StatusError(Exception):
    """Bot Status Error"""

    def __init__(self, *args: object) -> None:
        super().__init__(*args)


class ContainerNameError(Exception):
    """Bot Status Error"""

    def __init__(self, *args: object) -> None:
        super().__init__(*args)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='ddxl',
      version='0.0.5',
      description='DDXL',
      author='LcsH0s',
      author_email='decastro@et.esiea.fr',
      url='https://github.com/LcsH0s/ddxl',
      long_description=long_description,
      classifiers=["Development Status :: 2 - Pre-Alpha",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: MIT License",
                   "Operating System :: Unix",
                   "Programming Language :: Python :: 3.9",
                   ],
      packages=find_packages(exclude=["tests.*", "tests"]),
      install_requires=[
          'docker',
      ],
      )
